/*
** pork_acct.c - account management.
** Copyright (C) 2002-2003 Ryan McCabe <ryan@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <unistd.h>
#include <ncurses.h>
#include <string.h>
#include <sys/types.h>

#include <libfaim/aim.h>
#include <libfaim/faimconfig.h>

#include <pork.h>
#include <pork_missing.h>
#include <pork_util.h>
#include <pork_list.h>
#include <pork_buddy.h>
#include <pork_set.h>
#include <pork_swindow.h>
#include <pork_imwindow.h>
#include <pork_slist.h>
#include <pork_buddy_list.h>
#include <pork_acct.h>
#include <pork_misc.h>
#include <pork_input.h>
#include <pork_bind.h>
#include <pork_events.h>
#include <pork_screen.h>
#include <pork_screen_cmd.h>
#include <pork_aim.h>
#include <pork_conf.h>

extern struct screen screen;

static int pork_acct_find_helper(void *l, void *r) {
	char *left = l;
	struct pork_acct *acct = r;

	return (strcasecmp(left, acct->username));
}

/*
** Free everything that needs freeing in the
** given "aim_priv" structure.
*/

static void pork_priv_free(struct aim_priv *priv) {
	aim_kill_all_conn(priv);

	save_user_config(priv, priv->username);

	if (priv->blist != NULL) {
		blist_destroy(priv->blist);
		free(priv->blist);
	}

	buddy_destroy(priv);

	if (priv->passwd != NULL) {
		memset(priv->passwd, 0, strlen(priv->passwd));
		free(priv->passwd);
	}

	if (priv->away_msg != NULL) {
		free(priv->away_msg);
		hash_destroy(&priv->autoreply);
	}

	free(priv->username);
	free(priv->proxy);
	free(priv->proxy_user);
	free(priv->proxy_passwd);
	free(priv->server);
	free(priv->profile);

	free(priv);
}

inline dlist_t *pork_acct_find(char *user) {
	return (dlist_find(screen.acct_list, user, pork_acct_find_helper));
}

inline struct aim_priv *pork_acct_get_priv(char *user) {
	dlist_t *node;
	struct pork_acct *acct;

	node = dlist_find(screen.acct_list, user, pork_acct_find_helper);
	if (node == NULL || node->data == NULL)
		return (NULL);

	acct = node->data;
	return (acct->priv);
}

struct pork_acct *pork_acct_add(char *user) {
	dlist_t *node;
	struct pork_acct *acct;
	char nname[NUSER_LEN];
	struct aim_priv *priv;

	normalize(user, nname, sizeof(nname));

	/*
	** Make sure this account isn't already
	** setup.
	*/
	node = pork_acct_find(nname);
	if (node != NULL)
		return (NULL);

	priv = pork_priv_init(nname);
	if (priv == NULL)
		return (NULL);

	acct = xcalloc(1, sizeof(*acct));
	acct->username = xstrdup(nname);
	acct->priv = priv;
	priv->can_connect = 1;

	if (priv->profile == NULL)
		priv->profile = xstrdup(DEFAULT_AIM_PROFILE);

	screen.acct_list = dlist_add_head(screen.acct_list, acct);
	return (acct);
}

void pork_acct_del(dlist_t *node) {
	struct pork_acct *acct;
	dlist_t *cur;

	acct = node->data;
	screen.acct_list = dlist_remove(screen.acct_list, node);

	if (acct->priv == screen.null_priv)
		screen.null_priv = NULL;

	cur = screen.window_list;
	if (cur != NULL) {
		do {
			struct imwindow *win = cur->data;

			if (win->priv == acct->priv) {
				if (win->blist_visible)
					imwindow_blist_hide(win);

				win->priv->ref_count--;
				win->priv = screen.null_priv;

				if (win->priv != NULL)
					win->priv->ref_count++;
			}

			cur = cur->next;
		} while (cur != screen.window_list);
	}

	/* This must always be the case. */
	if (acct->priv->ref_count == 0)
		pork_priv_free(acct->priv);
	else
		fprintf(stderr, "failed for %s", acct->priv->username);

	free(acct->username);
	free(acct);
}

void pork_acct_del_all(void) {
	dlist_t *cur = screen.acct_list;

	while (cur != NULL) {
		dlist_t *next = cur->next;

		pork_acct_del(cur);
		cur = next;
	}
}

void acct_update_blist_color(void *data, void *nothing __notused) {
	struct pork_acct *acct = data;
	struct aim_priv *priv = acct->priv;

	if (priv->blist != NULL)
		blist_draw(priv->blist);
}

void acct_update_blist_format(void *data, void *nothing __notused) {
	struct pork_acct *acct = data;
	struct aim_priv *priv = acct->priv;

	if (priv->blist != NULL)
		blist_changed_format(priv->blist);
}

static void __pork_acct_update(void *data, void *nothing __notused) {
	struct pork_acct *acct = data;
	struct aim_priv *priv = acct->priv;
	struct timeval tv;

	if (!priv->can_connect)
		return;

#if 0
	/*
	** When the transfer mode is set to queued, this
	** will run every half second, sending any queued
	** commands to the AIM server.
	*/

	gettimeofday(&tv, NULL);
	if (tv.tv_sec > priv->last_flush.tv_sec ||
		tv.tv_usec > priv->last_flush.tv_usec + 500000)
	{
		aim_tx_flushqueue(&priv->aim_session);
		memcpy(&priv->last_flush, &tv, sizeof(priv->last_flush));
	}
#endif

	if (!priv->connected)
		return;

	/*
	** We've got to keep track of AIM buddies' idle time
	** ourselves; the AIM server reports an idle time
	** for them only when there's a change (i.e. they
	** go idle, come back from idle or their client reports
	** a new idle time).  This routine will run once a minute
	** and increment the idle time of buddies where appropriate.
	*/

	gettimeofday(&tv, NULL);
	if (tv.tv_sec >= priv->last_update + 60) {
		aim_keepalive(priv);
		buddy_update_idle(priv);
		priv->last_update = tv.tv_sec;
		aim_cleansnacs(&priv->aim_session, 59);
	}

	if (priv->buddy_pref != NULL && priv->buddy_pref->report_idle &&
		!priv->marked_idle && opt_get_bool(OPT_REPORT_IDLE))
	{
		time_t time_diff = tv.tv_sec - priv->last_input;
		int idle_after = opt_get_int(OPT_IDLE_AFTER);

		if (idle_after > 0 && time_diff >= 60 * idle_after) {
			screen_status_msg("Marking you idle after %d minutes of inactivity",
				time_diff / 60);
			aim_set_idle(priv, time_diff);
		}
	}
}

inline void pork_acct_update(void) {
	dlist_iterate(screen.acct_list, __pork_acct_update, NULL);
}

/*
** Initialize an "aim_priv" structure for the
** account owned by the specified user.
** Each user account gets its own priv structure,
** which contains account specific data.
*/

struct aim_priv *pork_priv_init(const char *user) {
	struct aim_priv *priv;

	priv = xcalloc(1, sizeof(*priv));

	if (buddy_init(priv) != 0) {
		free(priv);
		return (NULL);
	}

	priv->username = xstrdup(user);
	priv->events = &screen.events;
	read_user_config(priv, priv->username);
	blist_init(priv);

	aim_setup(priv);
	time(&priv->last_input);

	return (priv);
}
