/*
** pork_html.c - functions for dealing with HTML
** Copyright (C) 2002-2003 Amber Adams <amber@ojnk.net>
** Copyright (C) 2002-2003 Ryan McCabe <ryan@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <unistd.h>
#include <ncurses.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>

#include <pork_missing.h>
#include <pork_util.h>
#include <pork_list.h>
#include <pork_input.h>
#include <pork_misc.h>
#include <pork_html.h>
#include <pork_color.h>

/*
** Strip any html tags from "str"
** Replace common "&*;" forms with their ascii representation.
*/

// This function uses xmalloc() to grab its memory and so it must be
// free()'d when you're done using it.

char *strip_html(char *str) {
	char *p;
	char *buf;
	char *ret;
	int ignore = 0;
	char *last_tag = NULL;

	buf = xmalloc(strlen(str) + 1);
	ret = buf;

	for (p = str ; *p != '\0' ; p++) {
		if (ignore == 1) {
			if (*p == '>') {
				ignore = 0;

				if (last_tag != NULL) {
					if (!strncasecmp(last_tag, "<br>", 4) ||
						!strncasecmp(last_tag, "<p>", 3))
					{
						*buf++ = '\n';
					}

					last_tag = NULL;
				}
			} else if (*p == '<')
				last_tag = p;

			continue;
		}

		if (*p == '<') {
			ignore = 1;
			last_tag = p;
			continue;
		} else if (*p == '\t' || *p == '\f') {
			*buf++ = ' ';
			continue;
		} else if (*p == '&') {
			char *q = strchr(p, ';');

			p++;
			if (q != NULL) {
				*q = '\0';

				if (!strcasecmp(p, "nbsp"))
					*buf++ = ' ';
				else if (!strcasecmp(p, "lt"))
					*buf++ = '<';
				else if (!strcasecmp(p, "gt"))
					*buf++ = '>';
				else if (!strcasecmp(p, "amp"))
					*buf++ = '&';
				else if (!strcasecmp(p, "quot"))
					*buf++ = '"';
				else if (!strcasecmp(p, "apos"))
					*buf++ = '\'';
				else if (!strcasecmp(p, "hyphen"))
					*buf++ = '-';
				else if (!strcasecmp(p, "comma"))
					*buf++ = ',';
				else if (!strcasecmp(p, "excl"))
					*buf++ = '!';
				else if (!strcasecmp(p, "iexcl"))
					*buf++ = 'i';
				else if (!strcasecmp(p, "percnt"))
					*buf++ = '%';
				else if (!strcasecmp(p, "num"))
					*buf++ = '#';
				else if (!strcasecmp(p, "ast") || !strcasecmp(p, "mul"))
					*buf++ = '*';
				else if (!strcasecmp(p, "commat"))
					*buf++ = '@';
				else if (!strcasecmp(p, "lsqb"))
					*buf++ = '[';
				else if (!strcasecmp(p, "rsqb"))
					*buf++ = ']';
				else if (!strcasecmp(p, "bsol"))
					*buf++ = '\\';
				else if (!strcasecmp(p, "lcub"))
					*buf++ = '{';
				else if (!strcasecmp(p, "plus"))
					*buf++ = '+';
				else if (!strcasecmp(p, "divide"))
					*buf++ = '/';
				else if (!strcasecmp(p, "rcub"))
					*buf++ = '}';
				else if (!strcasecmp(p, "colon"))
					*buf++ = ':';
				else if (!strcasecmp(p, "period"))
					*buf++ = '.';
				else if (!strcasecmp(p, "semi"))
					*buf++ = ';';
				else if (!strcasecmp(p, "copy")) {
					strcpy(buf, "(C)");
					buf += 3;
				} else if (!strcasecmp(p, "trade")) {
					strcpy(buf, "(TM)");
					buf += 3;
				} else if (!strcasecmp(p, "reg")) {
					strcpy(buf, "(R)");
					buf += 3;
				} else {
					*buf++ = *p;
					continue;
				}

				p = q;
				continue;
			}
		} else if (*p == '\r')
			*buf++ = '\n';
		else
			*buf++ = *p;
	}

	*buf++ = '\0';

	return (ret);
}

// Converts plaintext to HTML, including pork color codes.
//
// Do NOT call free() on the pointer that this function returns.

char *text2html(const char *in) {
	static char buf[8192];
	const char *tmp = NULL;
	u_int32_t i;
	u_int32_t s = 0;

	if (!strncasecmp(in, "<html>", 6)) {
		xstrncpy(buf, in, sizeof(buf));
		return (buf);
	}

	for (i = 0 ; in[i] != '\0' && s < sizeof(buf) - 1 ; i++ ) {
		switch (in[i]) {
			case '<':
				tmp = "&lt;";
				break;

			case '>':
				tmp = "&gt;";
				break;

			case '&':
				tmp = "&amp;";
				break;

			default:
				buf[s++] = in[i];
				break;
		}

		if (tmp != NULL) {
			u_int32_t j;

			for (j = 0 ; tmp[j] != '\0' && s < sizeof(buf) - 1 ; j++)
				buf[s++] = tmp[j];
			tmp = NULL;
		}
	}

	buf[s] = '\0';

	return (buf);
}
